Payture = (function (Payture) {

  Payture.__internal__.model = (function (model) {
    var getDefaults = function () {
      return {
        AppleDomainName: document.domain,
        MerchantIdentifier: 'merchant.com.payture.applepay',
        CurrencyCode: 'RUB',
        Elements: {
          ApplePayButton: document.getElementsByClassName('apple-pay')[0]
        },
        VisibilitySettings: {
          ApplePayButton: false
        }
      }
    }

    var performValidation = function (valURL) {
      return new Promise(function (resolve, reject) {
        $ph.ajax({
          url: '/api/ApplePayValidateMerchant',
          data: $ph.getQuery({
            Url: valURL,
            DomainName: model.options.AppleDomainName
          }),
          type: 'POST',
          timeout: 60000,
          success: function (res) {
            resolve(res);
          },
          error: function (res) {
            model.onError(res);
          }
        })
      });
    }

    var sendPaymentToken = function (paymentToken) {
      var options = model.options, json = JSON.stringify(paymentToken.paymentData);

      var b64string = '';
      if (window.btoa) {
        b64string = window.btoa(unescape(encodeURIComponent(json)));
      } else {
        b64string = base64.encode(unescape(encodeURIComponent(json)));
      }

      return new Promise(function (resolve, reject) {
        var queryData = {
          Json: true,
          PayToken: b64string,
          EChequeCustomerContact: options.FormValues.EChequeCustomerContact,
          Data: $ph.getQuery({
            Key: options.FormValues.Key
          })
        }

        if (options.UsingFormValues.AddCard) {
          queryData.AddCard = options.FormValues.AddCard === "True";
        }

        $ph.ajax({
          url: model.options.ApiPath + '/PaySubmitMobile',
          data: $ph.getQuery(queryData),
          type: 'POST',
          timeout: 60000,
          success: function (res) {
            resolve(res);
          },
          error: function (res) {
            reject(res);
          }
        })
      });
    }

    return $ph.extend(model, {
      applePayInit: function () {
        //model.options = model.createOptions(model.options, getDefaults());
        var options = Object.assign({}, model.options);
        model.options = model.createOptions( model.options, getDefaults(), options);
        if (window.ApplePaySession) {
          var merchantIdentifier = model.options.MerchantIdentifier,
            promise = ApplePaySession.canMakePaymentsWithActiveCard(merchantIdentifier);

          promise.then(function (canMakePayments) {
            if (canMakePayments) {
              model.setOption('VisibilitySettings.ApplePayButton', true);
            }
          });
        }
      },
      applePay: function () {
        var amount = parseInt(document.getElementsByName('Amount')[0].value) / 100,
          product = document.getElementsByName('Product')[0].value;
        var supportedNetworks = ['visa', 'masterCard'];
        if(ApplePaySession.supportsVersion(11)) {
          supportedNetworks.push('mir');
        }        
        var request = {
          currencyCode: model.options.CurrencyCode,
		      countryCode: 'RU',
          supportedNetworks: supportedNetworks,
          merchantCapabilities: ['supports3DS'],
          total: {
            label: product,
            amount: amount
          },
        }

        var session = new ApplePaySession(3, request);

        session.onvalidatemerchant = function (evt) {
          var promise = performValidation(evt.validationURL);
          promise.then(function (merchantSession) {
            session.completeMerchantValidation(merchantSession);
          });
        }

        session.onpaymentmethodselected = function (evt) {
          var newTotal = {
            type: 'final',
            label: product,
            amount: amount
          };
          var newLineItems = [{
            type: 'final',
            label: product,
            amount: amount
          }];

          session.completePaymentMethodSelection(newTotal, newLineItems);
        }

        session.onpaymentauthorized = function (evt) {
          var promise = sendPaymentToken(evt.payment.token);
          promise.then(function (data) {
            var status;
            if (data.Success) {
              status = ApplePaySession.STATUS_SUCCESS;
              session.completePayment(status);
              model.onSuccess(data);
            } else {
              status = ApplePaySession.STATUS_FAILURE;
              session.completePayment(status);
              model.setOption("FormValues.Key", data.Key);
              model.onError(data);
            }
          });
        }

        session.oncancel = function (evt) {
          model.setOption('PaymentType', 'card_pay');
        }

        session.begin();
      }
    })
  })(Payture.__internal__.model)


  Payture.__internal__.view = (function (view) {
    var model = null,
      options = null,
      elements = null;

    return $ph.extend(view, {
      applePayInit: function () {
        model = Payture.__internal__.model;
        options = model.options;
        elements = options.Elements;

      }
    })
  })(Payture.__internal__.view)

  return $ph.extend(Payture, {
    ApplePay: function () {
      this.__internal__.model.applePayInit();
      this.__internal__.view.applePayInit();
    }
  })

})(Payture)